import pygame
from pygame.locals import *

from pykitten import utils
from pykitten.gui import  ImgButton, StaticWidget
from pykitten.gui import TextLine as OldTextLine


def draw_border(surf, color=(0,0,255), width=2):
    w, h = surf.get_size()
    pygame.draw.lines(surf, color, True,
        ((0,0),(w-width,0),(w-width,h-width),(0,h-width)), width)

class TextLine(OldTextLine): #yellow text
    def __init__(self, text, fontsize=20, color=(255,242,0),
        bgcolor=(0,0,0), font="courier-bold"):
        OldTextLine.__init__(self, text, fontsize, (255,242,0),
            bgcolor, font)


class Score(TextLine):
    digits = 5

    def __init__(self, score):
        self.value = score
        self.str_value = self.score_to_string()
        TextLine.__init__(self, self.str_value)
        x, y = 10, 10
        self.set_pos((x, y))

    def score_to_string(self):
        zero = ""
        score = str(self.value)
        if len(score) < self.digits:
            zero = "0" * (self.digits - len(score))

        return zero + score

    def set_text(self, score):
        self.value = score
        self.str_value = self.score_to_string()
        self.update(self.str_value)


class HighlightButton(ImgButton):
    def __init__(self, img, himg, simg, callback, **kwargs):
        ImgButton.__init__(self, img, callback, **kwargs)
        self.base_img = img  #normal img
        self.high_img = himg   #on hover
        self.sel_img = simg  #when selected
        self.sel = False

    def on_hover(self):
        self.fill((0,0,0))
        self.blit(self.high_img, (0,0))
        ImgButton.on_hover(self)

    def select(self):
        self.fill((0,0,0))
        self.blit(self.sel_img, (0,0))
        self.sel = True

    def on_unhover(self):
        self.fill((0,0,0))
        if self.sel:
            self.blit(self.sel_img, (0,0))
        else:
            self.blit(self.base_img, (0,0))

    def deselect(self):
        self.sel = False
        self.fill((0,0,0))
        self.blit(self.base_img, (0,0))


class TxtButton(ImgButton):
    def __init__(self, txt, fntsize=15, callback=None, **kwargs):
        img = TextLine(" "+txt+" ", fontsize=fntsize)

        w, h = img.get_size()
        pygame.draw.lines(img, (0,0,255), True,
            ((0,0),(w-1,0),(w-1,h-1),(0,h-1)), 1)
        ImgButton.__init__(self, img, callback, **kwargs)


class FixedTextButton(ImgButton):
    def __init__(self, size, txt, callback, **kwargs):
        img = pygame.Surface(size)
        ImgButton.__init__(self, img, callback, **kwargs)
        self.bg = img.copy()
        self.text = txt
        self.draw_text(txt)

    def draw_text(self, txt):
        self.txt = txt
        txt_img = TextLine(txt, fontsize=15)
        pos = utils.center(txt_img.get_size(), self.get_size())
        self.blit(self.bg, (0,0))
        self.blit(txt_img, pos)

    def draw_border(self, color=(0,0,255)):
        w, h = self.get_size()
        pygame.draw.lines(self.bg, color, True,
            ((0,0),(w-2,0),(w-2,h-2),(0,h-2)), 2)
        self.draw_text(self.txt)


class TextEntry(StaticWidget):
    max = 13

    def __init__(self, size=(160,22), fontsize=20, text="", blinky="|"):
        self.text = text
        self.fsize = fontsize
        self.text_img = TextLine(self.text, fontsize=fontsize)
        self.blinky = TextLine(blinky, fontsize=fontsize)
        self.do_blink = False
        StaticWidget.__init__(self, size)
        self.timer = utils.Timer(20)

        self.changed = 1
        self.update()

    def rem_text(self):
        if self.text:
            self.text = self.text[:-1]
            self.changed = 1

    def add_text(self, char):
        if len(self.text) < self.max:
            self.text += char
            self.changed = 1

    def update(self):
        self.fill((0,0,0))
        self.text_img = TextLine(self.text, self.fsize)
        self.blit(self.text_img, (0,0))

        if self.timer.update():
            self.do_blink = not self.do_blink
            self.changed = 1

        if self.do_blink:
            pos = (self.text_img.get_size()[0], 0)
            self.blit(self.blinky, pos)

        if self.changed:
            self.changed = 0
            return True
        return False


class FlipImg(StaticWidget):
    def __init__(self, images, interval=40):
        StaticWidget.__init__(self, images[0].get_size())
        self.images = images
        self.cur = 0
        self.timer = utils.Timer(interval)
        self.blit(self.images[self.cur], (0,0))

    def update(self):
        if self.timer.update():
            self.cur += 1
            self.redraw()
            return True
        return False

    def redraw(self):
        if self.cur >= len(self.images):
            self.cur = 0
        self.blit(self.images[self.cur], (0,0))

    def set_images(self, images):
        self.cur = 0
        self.images = images
        self.redraw()
        self.timer.activate()



class SuperFlipButton(ImgButton):
    def __init__(self, img, images, callback, **kwargs):
        ImgButton.__init__(self, img.copy(), callback, **kwargs)
        self.normal = img
        self.anim = FlipImg(images, interval=4)

    def animate(self):
        if self.anim.update():
            self.blit(self.anim, (0,0))

    def on_unhover(self):
        ImgButton.on_unhover(self)
        self.blit(self.normal, (0,0))

    def on_release(self):
        self.on_unhover()
        ImgButton.on_release(self)


class ClickFlipButton(ImgButton):
    def __init__(self, img, img2, callback, **kwargs):
        ImgButton.__init__(self, img.copy(), callback, **kwargs)
        self.img1 = img
        self.img2 = img2

    def on_click(self):
        ImgButton.on_click(self)
        self.blit(self.img2, (0,0))

    def on_release(self):
        ImgButton.on_release(self)
        self.blit(self.img1, (0,0))



