'''GUI module

Buttons, Progressbars and other widgety things
'''

import pygame

import data
import utils
from engine import State

FONTNAME = "courier-bold"

class StaticWidget(pygame.Surface):
    """Basic static widget

    Handles drawing and positioning
    """
    def __init__(self, size):
        """StaticWidget constructor

        @param size: size of widget in pixels, i.e. (100, 20)
        """
        pygame.Surface.__init__(self, size)
        self.size = size
        self.rect = self.get_rect()

    def set_pos(self, pos):
        """set widget's position
        """
        self.rect.topleft = pos

    def get_pos(self):
        """get widget's position
        """
        return self.rect.topleft

    def draw(self, surf):
        """draw widget on given surface
        """
        surf.blit(self, self.get_pos())

class ImgButton(StaticWidget):
    """Image button

    A clickeable static image.
    """
    def __init__(self, img, callback, **kwargs):
        """ImgButton constructor

        @param img: button image
        @param callback: function to be called when button is clicked
        @param kwargs: keyword arguments passed to callback
        """
        StaticWidget.__init__(self, img.get_size())
        self.blit(img, (0,0))
        self.callback = callback
        self.kwargs = kwargs

        self.hover = False
        self.pressed = False

    def on_hover(self):
        """called when mouse hovers button
        """
        pass

    def on_unhover(self):
        """called when mouse doesn't hover button anymore
        """
        pass

    def on_click(self):
        """called when button is pressed down
        """
        pass

    def on_release(self):
        """called when button is released - calls callback
        """
        self.callback(**self.kwargs)


    def update(self):
        """check if button is clicked or (un-)hovered
        """
        mouseover = self.rect.collidepoint(pygame.mouse.get_pos())
        l_button = pygame.mouse.get_pressed()[0]

        if mouseover and not self.hover:  #mouse over button for first time
            self.hover = True
            self.on_hover()
        elif not mouseover and self.hover: #mouse not over button anymore
            self.hover = False
            self.on_unhover()

        if mouseover and l_button and not self.pressed: #left mouse button pressed
            self.pressed = True
            self.on_click()
        elif self.pressed and not l_button: #left mouse button released
            self.pressed = False
            self.on_release()



class ProgressBar(StaticWidget):
    """Plain progressbar widget

    Useful for healthbars or other progress information
    """
    def __init__(self, size, color, bgcolor):
        """ProgressBar constructor

        @param size: 2-valued tuple
        @param color: foreground color (the "filling")
        @param bgcolor: background color
        """
        StaticWidget.__init__(self, size)
        self.length = size[0]
        self.height = size[1]
        self.color = color
        self.bgcolor = bgcolor
        self.perc = 0

    def change(self, perc):
        """fill progressbar by perc%

        i.e. perc=0.5 to fill half of progressbar
        """
        if perc > 1:
            perc = 1
        elif perc < 0:
            perc = 0
        self.perc = perc
        self.fill(self.bgcolor)
        filling = pygame.Surface((int(self.length*perc),self.height))
        filling.fill(self.color)
        self.blit(filling, (0,0))


class Cursor(pygame.sprite.Sprite):
    """An image that follows the mouse's position

    Useful to replace the default cursor. Don't forget to set
    pygame.mouse.set_visible(False) to hide the default cursor.
    """
    def __init__(self, image, clickpoint=(0,0)):
        """Cursor constructor

        @param image: cursor image
        @param clickpoint: Usually that's the topleft or topright pixel of the image.
        If you're using a crosshair as cursor, the clickpoint would probably
        be somewhere near the center.
        """
        pygame.sprite.Sprite.__init__(self)
        self.image = image
        self.base_image = image.copy()
        self.rect = self.image.get_rect()
        sx, sy = image.get_size()
        self.x_off, self.y_off = clickpoint
        self.update()

    def update(self):
        """follow mouse position
        """
        x, y = pygame.mouse.get_pos()
        self.rect.topleft = (x - self.x_off, y - self.y_off)

    def set_pos(self, pos):
        """set position of Mouse and Cursor
        """
        pygame.mouse.set_pos(pos)
        self.rect.topleft = pos


class ClickArea(pygame.Rect):
    """Like a button but without image, just a clickeable area
    """
    def __init__(self, area, callback, **kwargs):
        """ClickArea constructor

        @param area: [x_pos, y_pos, width, height]
        @param callback: function to be called on clicking the area
        @param kwargs: keyword arguments passed to callback
        """
        pygame.Rect.__init__(self, area)
        self.rect = self
        self.callback = callback
        self.kwargs = kwargs
        self.pressed = False
        self.hover = False

    def on_hover(self):
        """called when mouse hovers button
        """
        pass

    def on_unhover(self):
        """called when mouse doesn't hover button anymore
        """
        pass

    def on_click(self):
        """called when button is pressed down
        """
        pass

    def on_release(self):
        """called when button is released - calls callback
        """
        self.callback(**self.kwargs)


    def update(self):
        """check if button is clicked or (un-)hovered
        """
        mouseover = self.rect.collidepoint(pygame.mouse.get_pos())
        l_button = pygame.mouse.get_pressed()[0]

        if mouseover and not self.hover:  #mouse over button for first time
            self.hover = True
            self.on_hover()
        elif not mouseover and self.hover: #mouse not over button anymore
            self.hover = False
            self.on_unhover()
        elif mouseover and l_button and not self.pressed: #left mouse button pressed
            self.pressed = True
            self.on_click()
        elif self.pressed and not l_button: #left mouse button released
            self.pressed = False
            self.on_release()



class TextLine(StaticWidget):
    """A line of text

    Useful for one-liners like "Game Over."
    """
    def __init__(self, text, fontsize=20, color=(255,255,255), \
        bgcolor=(0,0,0), font=FONTNAME):
        font = data.filepath(font)
        self.font = pygame.font.Font(font, fontsize)
        self.color = color
        self.bgcolor = bgcolor
        img = self.font.render(text, 1, color, bgcolor)
        StaticWidget.__init__(self, img.get_size())
        self.blit(img, (0,0))

    def update(self, text):
        """change text
        """
        img = self.font.render(text, 1, self.color, self.bgcolor)
        self.fill(self.bgcolor)
        self.blit(img, (0,0))

class TextArea(pygame.Surface):
    """An area of text. Long lines will be wrapped.
    """
    def __init__(self, text, fontsize=15, length=45, font=FONTNAME, \
        color=(255,255,255), bgcolor=(0,0,0)):
        """TextArea constructor

        @param length: max number of characters per line
        """
        font = data.filepath(font)
        font = pygame.font.Font(font, fontsize)
        lineheight = fontsize + 4

        # generate textlines
        words = text.split(" ")
        line = ""
        lines = []
        for w in words:
            if len(line+w) <= length:
                line = line + w + " "
            else:
                lines.append(line)
                line = w + " "
        if line:
            lines.append(line)

        # generate images for textlines
        width = 0
        self.lines = []
        for line in lines:
            img = font.render(line, 1, color, bgcolor)
            self.lines.append(img)
            w = img.get_size()[0]
            if w > width:
                width = w

        # blit imagelines
        size = (width, len(self.lines)*lineheight)
        pygame.Surface.__init__(self, size)
        self.fill(bgcolor)
        y = 0
        for line in self.lines:
            self.blit(line, (0, y))
            y += lineheight


